﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MCP2221;

namespace ALScalib
{
    public class ADPS9300
    {
        private const byte ADDR_W = 0x72;
        private const byte ADDR_R = 0x73;
        private const int I2CSPEED = 100000;


        public enum Registers : byte
        {
            Control = 0x00,
            Timing = 0x01,
            Interrupt = 0x06,
            Id = 0x0A,
            Data0 = 0x0C,
            Data0lo = 0x0C,
            Data0hi = 0x0D,
            Data1 = 0x0E,
            Data1lo = 0x0E,
            Data1hi = 0x0F,
        }

        private const byte REG_CONTROL_CMD = 1<<7;
	    private const byte REG_CONTROL_IRQCLEAR = 1<<6;
	    private const byte REG_CONTROL_WORD = 1<<5;
        private const byte REG_CONTROL_POWER_ON = 0x03;

        /*public enum Gain : byte
        {
            G1 = 0,
            G16 = 1,
        }*/

        public enum IntegrationTime : byte
        {
            I13_7ms = 0,
            I101ms = 1,
            I402ms = 2,
        }

        IntegrationTime currentIntegrationTime;
        bool currentGain;


        MCP2221.MchpUsbI2c usbif = null;
        DevFunctions_M i2c = null;

        public ADPS9300()
        {
            usbif = new MchpUsbI2c();
            usbif.Management.SelectDev(0);
            i2c = usbif.Functions;
            
        }

        public void SetPower(bool on)
        {
            byte regval = 0;
            if (on)
            {
                regval |= REG_CONTROL_POWER_ON;
            }

            WriteRegister(Registers.Control, new byte[] { regval });

        }

        public void Config(bool highGain, IntegrationTime integrationTime)
        {
            byte regval = 0;

            regval |= (byte)((highGain ? 1 : 0) << 4);
            currentGain = highGain;
            this.currentIntegrationTime = integrationTime;
            

            regval |= (byte)integrationTime;

            Console.WriteLine("new config: 0x{0:X2}", regval);

            byte[] data = new byte[] { (byte)Registers.Timing | REG_CONTROL_CMD, regval };

            i2c.WriteI2cData(ADDR_W, data, 2, I2CSPEED);
        }

        public void WriteRegister(Registers reg, byte[] data)
        {
            byte[] d = new byte[data.Length + 1];

            d[0] = (byte)((byte)reg | REG_CONTROL_CMD);

            if (data.Length > 0)
            {
                Array.Copy(data, 0, d, 1, data.Length);
            }

            i2c.WriteI2cData(ADDR_W, d, (uint)d.Length, I2CSPEED);
        }

        public byte ReadRegisterByte(Registers reg)
        {
            i2c.WriteI2cData(ADDR_W, new byte[] { (byte)((byte)reg | REG_CONTROL_CMD) }, 1, I2CSPEED);
            byte[] rd = new byte[1];
            i2c.ReadI2cData(ADDR_R, rd, 1, I2CSPEED);
            return rd[0];
        }

        public UInt16 ReadRegisterWord(Registers reg)
        {
            i2c.WriteI2cData(ADDR_W, new byte[] { (byte)((byte)reg | REG_CONTROL_CMD | REG_CONTROL_WORD) }, 1, I2CSPEED);
            byte[] rd = new byte[2];
            i2c.ReadI2cData(ADDR_R, rd, 2, I2CSPEED);
            return (UInt16)((rd[1] << 8) | rd[0]);
        }

        public class SensorData
        {
            public UInt16 Ch0;
            public UInt16 Ch1;
            public IntegrationTime IntegrationTime;
            public bool HighGain;

            public double Luminanceold
            {
                get
                {
                    double ch0 = Ch0;
                    double ch1 = Ch1;

                    if (HighGain)
                    {
                        ch0 /= 16;
                        ch1 /= 16;
                    }

                    double divisor = 1;

                    switch (IntegrationTime)
                    {
                        case IntegrationTime.I13_7ms:
                            divisor = (double)11/322;
                            break;
                        case IntegrationTime.I101ms:
                            divisor = (double)81/322;
                            break;
                    }

                    ch0 /= divisor;
                    ch1 /= divisor;


                    if (ch0 == 0)
                    {
                        return 0;
                    }

                    if (ch0 == 65535 || ch1 == 65535)
                    {
                        return double.NaN;
                    }

                    double d = ch1 / ch0;
                    double lux = 0;

                    if (d > 0 && d <= 0.52)
                    {
                        lux = 0.0315 * ch0 - 0.0593 * ch0 * Math.Pow(d, 1.4);
                    }
                    else if (d > 0.52 && d <= 0.65)
                    {
                        lux = 0.0229 * ch0 - 0.0291 * ch1;
                    }
                    else if (d > 0.65 && d <= 0.80)
                    {
                        lux = 0.0157 * ch0 - 0.0180 * ch1;
                    }
                    else if (d > 0.80 && d <= 1.3)
                    {
                        lux = 0.00338 * ch0 - 0.00260 * ch1;
                    }
                    else if (d > 1.3)
                    {
                        lux = 0;
                    }

                    return lux*100;
                }
            }

            public double Luminance
            {
                get
                {
                    double ch0 = Ch0;
                    double ch1 = Ch1;

                    if (HighGain)
                    {
                        ch0 /= 16;
                        ch1 /= 16;
                    }

                    double divisor = 0.199922797307379;

                    switch (IntegrationTime)
                    {
                        case IntegrationTime.I13_7ms:
                            divisor *= 1;
                            break;
                        case IntegrationTime.I101ms:
                            divisor *= (double)101 / 13.7;
                            break;
                        case IntegrationTime.I402ms:
                            divisor *= (double)402 / 13.7;
                            break;
                    }

                    ch0 /= divisor;
                    ch1 /= divisor;


                    if (ch0 == 0)
                    {
                        return 0;
                    }

                    if (ch0 == 65535 || ch1 == 65535)
                    {
                        return double.NaN;
                    }

                    double d = ch1 / ch0;
                    double lux = 0;

                    if (d > 0 && d <= 0.52)
                    {
                        lux = 0.0315 * ch0 - 0.0593 * ch0 * Math.Pow(d, 1.4);
                    }
                    else if (d > 0.52 && d <= 0.65)
                    {
                        lux = 0.0229 * ch0 - 0.0291 * ch1;
                    }
                    else if (d > 0.65 && d <= 0.80)
                    {
                        lux = 0.0157 * ch0 - 0.0180 * ch1;
                    }
                    else if (d > 0.80 && d <= 1.3)
                    {
                        lux = 0.00338 * ch0 - 0.00260 * ch1;
                    }
                    else if (d > 1.3)
                    {
                        lux = 0;
                    }

                    return lux * 100;
                }
            }


            public SensorData(UInt16 ch0, UInt16 ch1, IntegrationTime integrationTime, bool highGain)
            {
                this.Ch0 = ch0;
                this.Ch1 = ch1;
                this.IntegrationTime = integrationTime;
                this.HighGain = highGain;
            }

        }

        public SensorData ReadSensordata()
        {
            UInt16 Ch0 = ReadRegisterWord(Registers.Data0);
            UInt16 Ch1 = ReadRegisterWord(Registers.Data1);

            SensorData retval = new SensorData(Ch0, Ch1, currentIntegrationTime, currentGain);

            


            return retval;
        }


    }
}
